/*
 * 
 * 
 */

package autofeature.generator;

import autofeature.intermediate.Automaton;
import autofeature.intermediate.SpecificationException;
import autofeature.intermediate.Specification;
import autofeature.intermediate.State;
import autofeature.intermediate.Event;
import autofeature.intermediate.Shadow;
import java.util.ArrayList;
import java.util.List;
import org.json.simple.JSONObject;

/**
 *
 * @author Hendrik Speidel - speidel@fmi.uni-passau.de
 */
public class ACCGenerator implements Generator {

    private final JSONObject meta;

    String COMMENT_START = "/* ";
    String COMMENT_END = " */";
    String STATE_ENUM_FORMAT = "__STATE__%s";
    String CURRENT_STATE_FORMAT = "__CURRENT_STATE__%s";


    public ACCGenerator(JSONObject meta) {
        this.meta = meta;
    }

    public void transform(Specification spec) {

        //gather introductions
        //String intro = spec.getIntroduction();

        //write(intro + "\n");
        write("#include \"wsllib.h\"\n");

        for (Automaton a : spec.getAutomata()) {
            if (!a.getStates().isEmpty()) {
                processAutomatonStateIntro(a);
                
            }
            processAutomatonIntro(a);
        }

        for (Automaton a : spec.getAutomata()) {
            processAutomaton(a);
        }

    }

    public void write(String m) {
        System.out.print(m);
    }

    public void writeComment(String m) {
        write(COMMENT_START + m + COMMENT_END + "\n");
    }

    private void processAutomatonStateIntro(Automaton automaton) {

        writeComment(String.format("States of Automaton '%s'",automaton.getName()));
        String stateEnum = String.format(STATE_ENUM_FORMAT, automaton.getName());
        String currentState = String.format(CURRENT_STATE_FORMAT, automaton.getName());
        write("enum ");
        write(stateEnum);
        write(" {\n");

        List<State> states = automaton.getStates();
        for (int i = 0; i < states.size(); i++) {
            if (i > 0) {
                write(", ");
            }
            write(states.get(i).getName());
        }

        write("};\n");
        write("enum ");
        write(stateEnum);
        write(" ");
        write(currentState);
        write(" = ");
        write(states.get(0).getName());
        write(";\n");

    }

    private void processAutomaton(Automaton automaton) {

        writeComment(automaton.getName());        
        List<Event> globalEvents = automaton.getGlobalEvents();
        List<State> states = automaton.getStates();

        if (!globalEvents.isEmpty() && !states.isEmpty()) {
            throw new SpecificationException(
                    String.format("Automaton '%s' contains both " +
                    "global events and states. This is not allowed!",
                    automaton.getName()));

        }

        //process global events
        for (Event e: globalEvents) {
            processEvent(null, e);
        }

        //

        //process states
        for (State s: states) {
            processState(s);
        }


    }

    private void processEventFuncParams(Event event) {
        List<String> types = event.getParameterTypes();
        
        boolean first = true;

        for (String type : types) {
            if (first) {
                first = false;
            } else {
                write(", ");
            }
            write(type);
        }
    }

    private void processAroundParams(Event event) {
        //TODO result for after

        List<String> types = event.getParameterTypes();
        List<String> names = event.getParameterNames();
        boolean first = true;

        if (!event.isBefore() && event.getReturnValueName() != null) {
            write(event.getReturnType() + " " + event.getReturnValueName());
            first = false;
        }

        for (int i = 0; i < types.size(); i++) {
            String type = types.get(i);
            String name = names.get(i);
            if (!name.equals("_")) {
                if (first) {
                    first = false;
                } else {
                    write(", ");
                }
                write(type + " " + name);
            }
        }
    }

    private void processEvent(State state, Event event) {
        writeComment("Event " + event.getFunctionName());

        if (event.isBefore()) {
            write("before(");
        } else {
            write("after(");
        }

        //write(event.getReturnType());
        //write(" ");
        //write("around(");
        processAroundParams(event);
        write("): ");
        write("execution(");
        write(event.getReturnType() + " ");
        write(event.getFunctionName() + "(");
        processEventFuncParams(event);
        write(")) ");
        processEventArgs(event);
        write("{\n");

        //if (!event.isBefore()) {
        //    write("\tproceed();\n");
        //}

        if (state != null) {
            write("\tif (");
            write(String.format(CURRENT_STATE_FORMAT, state.automaton.getName()));
            write(" == ");
            write(state.getName());
            write(") {\n");
        }

        String condition = event.getCondition();
        if (condition != null && !condition.equals("")) {
            write("\tif (");
            write(condition);
            write(") {\n");
        }

        write(event.getAction());
        write("\n");

        if (condition != null && !condition.equals("")) {
            write("\t}\n");
        }
        
        if (state != null) {
            write("\t}\n");
        }

        //if (event.isBefore()) {
        //    write("\tproceed();\n");
        //}
        write("}\n");
    }

    private void processEventArgs(Event event) {
        List<String> types = event.getParameterTypes();
        List<String> names = event.getParameterNames();
        boolean first = true;
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < types.size(); i++) {
            String type = types.get(i);
            String name = names.get(i);
            if (first) {
                first = false;
            } else {
                buf.append(", ");
            }
            if (name.equals("_")) {
                buf.append(type);
            } else {
                buf.append(name);
            }
        }
        if (!first) {
            //parameters bound -> write args(...)
            write("&& args(");
            write(buf.toString());
            write(") ");
        }

        if (!event.isBefore() && event.getReturnValueName() != null) {
            write("&& result(");
            write(event.getReturnValueName());
            write(") ");
        }
    }

    private void processState(State s) {
        writeComment(String.format("State '%s'", s.getName()));
        for(Event event : s.getEvents()) {
            processEvent(s, event);
        }
    }

    private void processAutomatonIntro(Automaton a) {
        write(a.getComment());
        writeComment(String.format("Introductions of Automaton '%s'", a.getName()));
        write(a.getIntroduction());

        List<Shadow> shadows = a.getShadows();

        for (Shadow s : shadows) {
            write("introduce(): intype(");
            write(s.getName());
            write(") {\n");
            write(s.getIntroductions());
            write("\n}\n\n");

        }
        
        write("\n\n");


    }



}
