/*
 * 
 * 
 */

package cpawsl.parser;
import autofeature.parser.SpecificationReader;
import autofeature.intermediate.Automaton;
import autofeature.intermediate.Event;
import autofeature.intermediate.Specification;
import autofeature.intermediate.State;
import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.antlr.runtime.RecognitionException;
import org.junit.Test;
import static org.junit.Assert.*;
/**
 *
 * @author Hendrik Speidel - speidel@fmi.uni-passau.de
 */
public class AutomatonTest {
    Logger logger = Logger.getLogger(AutomatonTest.class.getName());

    String TEST_DIR = "/export/home/henne/uni/cpawsl/test/cpawsl/parser/automaton-tests/";

    private Specification load(String filename) throws IOException, RecognitionException {
        return new SpecificationReader().readFile(TEST_DIR + filename + ".spec");
    }


    @Test
    public void testEmptySpec() throws Exception {

        Specification spec = load("empty_spec");
        assertEquals("empty spec should have \"\" as introduction!", "", spec.getIntroduction());
        assertTrue("spec should have no automata!", spec.getAutomata().isEmpty());

    }

    @Test
    public void testEmptyAutomaton() throws Exception {

        Specification spec = load("empty_automaton");
        assertEquals("spec should have \"\" as introduction!", "", spec.getIntroduction());
        assertEquals("spec should have one automaton!", 1, spec.getAutomata().size());
        Automaton a = spec.getAutomata().get(0);
        assertEquals("automaton should be named Test1!", "Test1", a.getName());
        assertTrue("automaton should have no global events!", a.getGlobalEvents().isEmpty());
        assertTrue("automaton should have no states!", a.getStates().isEmpty());
        assertEquals("automaton should have \"\" as introduction!", "", a.getIntroduction());
    }

    @Test
    public void testAutomatonIntroduction() throws Exception {

        Specification spec = load("automaton_intro_decl");
        assertEquals("spec should have \"\" as introduction!", "", spec.getIntroduction());
        assertEquals("spec should have one automaton!", 1, spec.getAutomata().size());
        Automaton a = spec.getAutomata().get(0);
        assertEquals("automaton should be named Test2!", "Test2", a.getName());
        assertTrue("automaton should have no states!", a.getStates().isEmpty());
        assertTrue("automaton should have no global events!", a.getGlobalEvents().isEmpty());
        assertEquals("automaton should have \"int i;\" as introduction!", "int i;", a.getIntroduction());

    }

    @Test
    public void testEventBefore() throws Exception {

        Specification spec = load("automaton_event_before");
        assertEquals("spec should have \"\" as introduction!", "", spec.getIntroduction());
        assertEquals("spec should have one automaton!", 1, spec.getAutomata().size());
        Automaton a = spec.getAutomata().get(0);
        assertEquals("automaton should be named Test3!", "Test3", a.getName());
        assertTrue("automaton should have no states!", a.getStates().isEmpty());
        assertEquals("automaton should have \"\" as introduction!", "", a.getIntroduction());
        assertEquals("automaton should have 5 global events!", 5, a.getGlobalEvents().size());

        
        List<Event> evs = a.getGlobalEvents();
        assertTrue("First event should be a before", evs.get(0).isBefore());
        assertEquals("First event should be called main", "main", evs.get(0).getFunctionName());
        assertEquals("First event should return void", "void", evs.get(0).getReturnType());
        assertTrue("First event should take no params", evs.get(0).getParameterTypes().isEmpty());
        assertNull("First event should have empty condition", evs.get(0).getCondition());
        assertEquals("First event should have empty action", "", evs.get(0).getAction());


        assertTrue("2nd event should be a before", evs.get(1).isBefore());
        assertEquals("2nd event should be called main2", "main2", evs.get(1).getFunctionName());
        assertEquals("2nd event should return void", "void", evs.get(1).getReturnType());
        assertEquals("2nd event should take one params", 1, evs.get(1).getParameterTypes().size());
        assertEquals("2nd event 1st param should be _", "_", evs.get(1).getParameterNames().get(0));
        assertEquals("2nd event 1st param should be int", "int", evs.get(1).getParameterTypes().get(0));
        
        assertTrue("3rd event should be a before", evs.get(2).isBefore());
        assertEquals("3rd event should be called main3", "main3", evs.get(2).getFunctionName());
        assertEquals("3rd event should return void", "void", evs.get(2).getReturnType());
        assertEquals("3rd event should take 3 params", 3, evs.get(2).getParameterTypes().size());
        assertEquals("3rd event 3rd param should be c", "c", evs.get(2).getParameterNames().get(2));
        assertEquals("3rd event 3rd param should be struct email *", "struct email *", evs.get(2).getParameterTypes().get(2));

        assertTrue("4th event should be a before", evs.get(3).isBefore());
        assertEquals("4th event should be called main4", "main4", evs.get(3).getFunctionName());
        assertEquals("4th event should return struct email *", "struct email *", evs.get(3).getReturnType());
        

        assertTrue("5th event should be a before", evs.get(4).isBefore());
        assertEquals("5th event should be called main5", "main5", evs.get(4).getFunctionName());
        assertEquals("5th event should return void", "void", evs.get(4).getReturnType());
        assertEquals("5th event should have \"int i=7;i++;\" action", "int i=7;i++;", evs.get(4).getAction());

    }

    
    @Test
    public void testStateExtension() throws Exception {

        Specification spec = load("automaton_state_extension");
        assertEquals("spec should have \"\" as introduction!", "", spec.getIntroduction());
        assertEquals("spec should have one automaton!", 1, spec.getAutomata().size());
        Automaton a = spec.getAutomata().get(0);
        assertEquals("automaton should be named TestStates!", "TestStates", a.getName());
        assertEquals("automaton should have 2 states!", 2 , a.getStates().size());
        assertEquals("automaton should have \"\" as introduction!", "", a.getIntroduction());
        assertEquals("automaton should have no global events!", 0, a.getGlobalEvents().size());

        List<State> states = a.getStates();
        assertEquals("INIT", states.get(0).getName());
        assertEquals(1, states.get(0).getEvents().size());
        assertEquals("TEST", states.get(1).getName());
        assertEquals(1, states.get(1).getEvents().size());
    }


    @Test
    public void testEventAfter() throws Exception {

        Specification spec = load("automaton_event_after");
        assertEquals("spec should have \"\" as introduction!", "", spec.getIntroduction());
        assertEquals("spec should have one automaton!", 1, spec.getAutomata().size());
        Automaton a = spec.getAutomata().get(0);
        assertEquals("automaton should be named Test4!", "Test4", a.getName());
        assertTrue("automaton should have no states!", a.getStates().isEmpty());
        assertEquals("automaton should have \"\" as introduction!", "", a.getIntroduction());
        assertEquals("automaton should have 5 global events!", 5, a.getGlobalEvents().size());


        List<Event> evs = a.getGlobalEvents();
        assertFalse("First event should be a after", evs.get(0).isBefore());
        assertEquals("First event should be called main", "main", evs.get(0).getFunctionName());
        assertEquals("First event should return int", "int", evs.get(0).getReturnType());
        assertTrue("First event should take no params", evs.get(0).getParameterTypes().isEmpty());
        assertNull("First event should have empty condition", evs.get(0).getCondition());
        assertEquals("First event should have empty action", "", evs.get(0).getAction());
        assertNull("should not have a returnvalue", evs.get(0).getReturnValueName());

        assertFalse("2nd event should be a after", evs.get(1).isBefore());
        assertEquals("2nd event should be called main2", "main2", evs.get(1).getFunctionName());
        assertEquals("2nd event should return int", "int", evs.get(1).getReturnType());
        assertEquals("2nd event should take one params", 1, evs.get(1).getParameterTypes().size());
        assertEquals("2nd event 1st param should be _", "_", evs.get(1).getParameterNames().get(0));
        assertEquals("2nd event 1st param should be int", "int", evs.get(1).getParameterTypes().get(0));
        assertEquals("should not have 'x' returnvalue", "x", evs.get(1).getReturnValueName());

        assertEquals("3rd event should be called main3", "main3", evs.get(2).getFunctionName());
        assertEquals("3rd event should return int", "int", evs.get(2).getReturnType());
        assertEquals("3rd event should take 3 params", 3, evs.get(2).getParameterTypes().size());
        assertEquals("3rd event 3rd param should be c", "c", evs.get(2).getParameterNames().get(2));
        assertEquals("3rd event 3rd param should be struct email *", "struct email *", evs.get(2).getParameterTypes().get(2));
        assertEquals("should not have 'x' returnvalue", "x", evs.get(2).getReturnValueName());

        assertEquals("4th event should be called main4", "main4", evs.get(3).getFunctionName());
        assertEquals("4th event should return struct email *", "struct email *", evs.get(3).getReturnType());
        assertEquals("should not have 'x' returnvalue", "x", evs.get(3).getReturnValueName());

        assertEquals("5th event should be called main5", "main5", evs.get(4).getFunctionName());
        assertEquals("5th event should return int", "int", evs.get(4).getReturnType());
        assertEquals("5th event should have \"int i=7;i++;\" action", "int i=7;i++;", evs.get(4).getAction());
        assertEquals("should not have 'x' returnvalue", "x", evs.get(4).getReturnValueName());
    }


}
